@@ -31,16 +31,10 @@ CGI::Application::Plugin::Authentication::Driver::DBI - DBI Authentication drive
 
 This Authentication driver uses the DBI module to allow you to authenticate against
 any database for which there is a DBD module.  You can either provide an active
-database handle, or provide the parameters necessary to connect to the database.
+database handle, or provide the parameters necesary to connect to the database.
 
 When describing the database structure, you need to specify some or all of the
-following parameters: TABLE(S), JOIN_ON, COLUMNS, CONSTRAINTS, ORDER_BY and
-LIMIT.
-
-=head2 DBH
-
-The DBI database handle to use. Defaults to C<$self->dbh()>, which is provided and configured
-through L<CGI::Application::Plugin::DBH|CGI::Application::Plugin::DBH>
+following parameters: TABLE(S), JOIN_ON, COLUMNS and CONSTRAINTS.
 
 =head2 TABLE(S)  (required)
 
@@ -74,12 +68,12 @@ username = ?), but some parameters may need to be tested locally in perl, so the
 must be listed in the COLUMNS option.  One example of a value that needs to be tested
 in perl is a crypted password.  In order to test a crypted password, you need to
 take the entered password, and crypt it with the salt of the already crypted password.
-But until we actually see the password that is in the database, we will not know the
+But until we actaully see the password that is in the database, we will not know the
 value of the salt that was used to encrypt the password.  So we pull the value out
 using COLUMNS, and the test will be performed automatically in perl.
 
 Any value that matches __CREDENTIAL_n__ (where n is a number) will be replaced with
-the corresponding credential that was entered by the user.  For an explanation of
+the coresponding credential that was entered by the user.  For an explanation of
 what the credentials are and where they come from, see the section headed with
 CREDENTIALS in L<CGI::Application::Plugin::Authentication>.
 
@@ -93,7 +87,7 @@ will be added to the WHERE clause of the SQL query, and will ideally reduce
 the number of returned rows to one.  
 
 Any value that matches __CREDENTIAL_n__ (where n is a number) will be replaced with
-the corresponding credential that was entered by the user.  For an explanation of
+the coresponding credential that was entered by the user.  For an explanation of
 what the credentials are and where they come from, see the section headed with
 CREDENTIALS in L<CGI::Application::Plugin::Authentication>.
 
@@ -104,27 +98,6 @@ CREDENTIALS in L<CGI::Application::Plugin::Authentication>.
      }
 
 
-=head2 ORDER_BY  (optional)
-
-This option allows you to order the result set, in case the query returns
-multiple rows.
-
-     ORDER_BY => 'created DESC'
-
-Note: This option is only useful if you also specify the COLUMNS option.
-
-=head2 LIMIT  (optional)
-
-In some situations your query may return multiple rows when you only want it to
-return one.  For example if you insert and date a new row instead of updating
-the existing row when the details for an account change.  In this case you want
-the newest record from the result set, so it will be important to order the
-result set and limit it to return only one row.
-
-     LIMIT => 1
-
-Note: This option is only useful if you also specify the COLUMNS option.
-
 =head1 ENCODED PASSWORDS
 
 It is quite common to store passwords in a database in some form that makes them hard
@@ -179,7 +152,7 @@ And here is what will happen behind the scenes:
 
 Again, this is all done automatically behind the scenes, but I've included it here to illustrate how
 the queries are performed, and how the comparisons are handled.  For more information
-see the section labelled ENCODED PASSWORDS in the L<CGI::Application::Plugin::Authentication::Driver>
+see the section labeled ENCODED PASSWORDS in the L<CGI::Application::Plugin::Authentication::Driver>
 docs.
 
 
@@ -191,7 +164,10 @@ docs.
  #  two separate tables.
  __PACKAGE__->authen->config(
      DRIVER => [ 'DBI',
-         # the handle comes from $self->dbh, via the "DBH" plugin. 
+         DSN         => 'dbi:Pg:dbname=test',
+         DB_USER     => 'postges',
+         DB_PASSWORD => 'secret',
+         DBI_OPTIONS => { RaiseError => 1, AutoCommit => 0 },
          TABLES      => ['user', 'domain'],
          JOIN_ON     => 'user.domainid = domain.id',
          CONSTRAINTS => {
@@ -210,7 +186,7 @@ docs.
  #  properly, we need to pull out the password, and check it locally
  __PACKAGE__->authen->config(
      DRIVER => [ 'DBI',
-         DBH         => $dbh,   # provide your own DBI handle
+         DBH         => $self->dbh,
          TABLE       => 'user',
          CONSTRAINTS => { 'user.name'      => '__CREDENTIAL_1__' }
          COLUMNS     => { 'crypt:password' => '__CREDENTIAL_2__' },
@@ -223,11 +199,12 @@ docs.
  #  Here we only check users where the 'active' column is true
  __PACKAGE__->authen->config(
      DRIVER => [ 'DBI',
+         DBH         => $self->dbh,
          TABLE       => 'user',
          CONSTRAINTS => {
              'user.name'     => '__CREDENTIAL_1__',
              'user.password' => '__CREDENTIAL_2__',
-             'user.active'   => 't'
+             'active'        => 't'
          },
      ],
  );
@@ -240,15 +217,16 @@ docs.
  #  an MD5 hash hex format and stored in upper case).
  __PACKAGE__->authen->config(
      DRIVER => [ 'DBI',
+         DBH         => $self->dbh,
          TABLES      => ['user U', 'dailycode D'],
          JOIN_ON     => 'U.userid = D.userid',
          CONSTRAINTS => {
-             'U.name'            => '__CREDENTIAL_1__',
+             'user.name'         => '__CREDENTIAL_1__',
              'uc:md5_hex:D.code' => '__CREDENTIAL_3__',
              'D.date'            => 'now'
          },
          COLUMNS     => {
-             'crypt:U.password' => '__CREDENTIAL_2__'
+             'crypt:password' => '__CREDENTIAL_2__'
          },
      ],
  );
@@ -273,29 +251,23 @@ sub verify_credentials {
     die "The DBI driver requires a hash of options" if @_options % 2;
     my %options = @_options;
 
-    # Get a database handle - either one that is given to us, or see if there
-    # is a ->dbh method in the CGIApp module (This is provided by the
-    # CGI::Application::Plugin::DBH module, so use it if it is there).
+    # Get a database handle either one that is given to us, or connect using the
+    # information given in the configuration
     my $dbh;
     if ( $options{DBH} ) {
         $dbh = $options{DBH};
-    } elsif ( $self->authen->_cgiapp->can('dbh') ) {
-        $dbh = $self->authen->_cgiapp->dbh;
+    } elsif ( $options{DSN} ) {
+        no warnings qw(uninitialized);
+        $dbh = DBI->connect( $options{DBH}, $options{DB_USER}, $options{DB_PASSWORD}, $options{DBI_OPTIONS} )
+          or die $DBI::errstr;
     } else {
-        die "No DBH handle passed to the DBI Driver, and no dbh() method detected";
+        die "No DBH or DSN parameter passed to the DBI Driver";
     }
 
     # Grab the database table names (TABLE and TABLES are synonymous)
     my $tables = $options{TABLES} || $options{TABLE};
-    die "No TABLE parameter defined" unless defined($tables);
     $tables = [$tables] unless ref $tables eq 'ARRAY';
 
-    # See if we need to order the result set
-    my $order_by = $options{ORDER_BY} ? ' ORDER BY '.$options{ORDER_BY} : '';
-
-    # See if we need to limit the result set
-    my $limit = $options{LIMIT} ? ' LIMIT '.$options{LIMIT} : '';
-
     # Grab all the columns that we need to pull out.  We also grab a list of
     # columns that are stripped of any encoding information.
     # If no columns are provided we just select count(*) for efficiency.
@@ -337,8 +309,6 @@ sub verify_credentials {
     push @where, map { $_ . ' = ?' } keys %constraints;
     $sql .= join( ' AND ', @where );
     my @params = values %constraints;
-    $sql .= $order_by;
-    $sql .= $limit;
 
     # prepare and execute the SQL
     my $sth = $dbh->prepare_cached($sql) || die "Failed to prepare SQL statement:  " . $dbh->errstr;
