@@ -7,8 +7,7 @@ use base qw(CGI::Application::Plugin::Authentication::Store);
 
 sub initialize {
     my $self = shift;
-    my ($storage) = $self->options;
-    $self->{__DUMMY} = $storage || {};
+    $self->{__DUMMY} = {};
 }
 
 sub fetch {
